/**
    Copyright (C) 2003  Michael Ahlberg, Måns Rullgård

    Permission is hereby granted, free of charge, to any person
    obtaining a copy of this software and associated documentation
    files (the "Software"), to deal in the Software without
    restriction, including without limitation the rights to use, copy,
    modify, merge, publish, distribute, sublicense, and/or sell copies
    of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be
    included in all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
    EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
    NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
    HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
    WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.
**/

#define _LARGEFILE64_SOURCE

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <alloca.h>
#include <tctypes.h>
#include <tcstring.h>
#include <tcalloc.h>
#include <sys/stat.h>
#include <file_tc2.h>

extern int
file_read(void *buf, size_t size, size_t count, url_t *u)
{
    return fread(buf, size, count, u->private);
}

extern int
file_write(void *buf, size_t size, size_t count, url_t *u)
{
    return fwrite(buf, size, count, u->private);
}

extern int
file_seek(url_t *u, int64_t offset, int how)
{
    return fseeko64(u->private, offset, how);
}

extern uint64_t
file_tell(url_t *u)
{
    return ftello64(u->private);
}

extern int
file_close(url_t *u)
{
    FILE *f = u->private;
    int r = fclose(f);
    u->private = NULL;
    tcfree(u);
    return r;
}

static void
file_free(void *p)
{
    url_t *u = p;
    if(u->private)
	fclose(u->private);
}

extern url_t *
file_open(char *url, char *mode)
{
    struct stat64 st;
    FILE *f;
    url_t *u;

    if(!strncmp(url, "file:", 5))
	url += 5;

    if(stat64(url, &st)){
	if(*mode == 'r')
	    return NULL;
    } else if(S_ISDIR(st.st_mode)){
	return dir_open(url, mode);
    }

    if(!(f = fopen64(url, mode)))
	return NULL;

    u = tcallocdz(sizeof(*u), NULL, file_free);
    if(strchr(mode, 'r'))
	u->read = file_read;
    if(strchr(mode, 'w'))
	u->write = file_write;
    u->seek = file_seek;
    u->tell = file_tell;
    u->close = file_close;
    u->private = f;

    u->size = st.st_size;

    return u;
}
