#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <tctypes.h>
#include <tcstring.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <tcalloc.h>
#include <udp_tc2.h>

#define min(a,b) ((a)<(b)?(a):(b))
#define max(a,b) ((a)>(b)?(a):(b))

#define buffer_size tc2_url_udp_conf_buffer_size

typedef struct mc_stream {
    int socket;
    struct sockaddr_in addr;
    uint64_t pos;
    char *buf;
    int bpos, bsize;
} mc_stream_t;

extern int
mc_read(void *buf, size_t size, size_t count, url_t *u)
{
    mc_stream_t *mcs = u->private;
    int s = size * count;

    while(s){
	int ss;

	if(mcs->bsize == 0){
	    struct sockaddr a;
	    int r, l = sizeof(a);
	    mcs->bpos = 0;
	    r = recvfrom(mcs->socket, mcs->buf, 65536, 0, &a, &l);
	    if(r < 0)
		break;
	    mcs->bsize += r;
	}

	ss = min(s, mcs->bsize);
	ss = min(ss, buffer_size - mcs->bpos);
	memcpy(buf, mcs->buf + mcs->bpos, ss);
	buf += ss;
	mcs->bpos += ss;
	mcs->bsize -= ss;
	mcs->pos += ss;
	s -= ss;

	if(mcs->bpos == buffer_size)
	    mcs->bpos = 0;
    }

    s = size * count - s;
    return s / size;
}

extern int
mc_write(void *buf, size_t size, size_t count, url_t *u)
{
    mc_stream_t *mcs = u->private;
    int s;

    s = sendto(mcs->socket, buf, size * count, 0,
	       (struct sockaddr *)&mcs->addr, sizeof(mcs->addr));
    if(s < 0)
	return s;

    mcs->pos += s;

    return s / size;
}

extern int
mc_seek(url_t *u, int64_t offset, int how)
{
    mc_stream_t *mcs = u->private;
    uint64_t p, dp;

    switch(how){
    case SEEK_SET:
	p = offset;
	break;
    case SEEK_CUR:
	p = mcs->pos + offset;
	break;
    default:
	return -1;
    }

    dp = p - mcs->pos;

    if(dp > mcs->bsize)
	return -1;

    if(dp < - mcs->bpos)
	return -1;

    mcs->bpos += dp;
    mcs->bsize -= dp;

    return 0;
}

extern uint64_t
mc_tell(url_t *u)
{
    mc_stream_t *mcs = u->private;
    return mcs->pos;
}

extern int
mc_close(url_t *u)
{
    mc_stream_t *mcs = u->private;

    close(mcs->socket);
    free(mcs->buf);
    tcfree(u);

    return 0;
}

extern url_t *
mc_open(char *url, char *mode)
{
    struct sockaddr_in sa;
    struct hostent *hn;
    mc_stream_t *mcs;
    uint32_t addr;
    url_t *u;
    char *pp;
    int port;
    int sock;
    int i;

    url = strdupa(url);

    if(!strncmp(url, "multicast:", 10))
	url += 10;
    else if(!strncmp(url, "udp:", 4))
	url += 4;

    while(*url == '/')
	url++;

    if(!(pp = strchr(url, ':')))
	return NULL;

    *pp++ = 0;
    port = strtol(pp, NULL, 0);

    if(!(hn = gethostbyname(url)))
	return NULL;

    if((sock = socket(PF_INET, SOCK_DGRAM, 0)) < 0)
	return NULL;

    i = 1;
    setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, &i, sizeof(i));

    sa.sin_family = AF_INET;
    sa.sin_port = htons(port);

    addr = *(uint32_t *) hn->h_addr;
    if(IN_MULTICAST(ntohl(addr))){
	struct ip_mreq mrq;
	mrq.imr_multiaddr.s_addr = addr;
	mrq.imr_interface.s_addr = htonl(INADDR_ANY);
	if(setsockopt(sock, IPPROTO_IP, IP_ADD_MEMBERSHIP, &mrq,
		      sizeof(mrq)) < 0)
	    return NULL;
	sa.sin_addr.s_addr = addr;
    } else {
	sa.sin_addr.s_addr = htonl(INADDR_ANY);
    }

    if(*mode == 'r'){
	if(bind(sock, (struct sockaddr *) &sa, sizeof(sa)) < 0)
	    return NULL;
    } else if(*mode == 'w'){
	sa.sin_addr.s_addr = addr;
    } else {
	return NULL;
    }

    mcs = calloc(1, sizeof(*mcs));
    mcs->socket = sock;
    mcs->addr = sa;

    mcs->buf = malloc(max(buffer_size, 65536));

    u = tcallocz(sizeof(*u));
    if(strchr(mode, 'r'))
	u->read = mc_read;
    if(strchr(mode, 'w'))
	u->write = mc_write;
    u->seek = mc_seek;
    u->tell = mc_tell;
    u->close = mc_close;
    u->private = mcs;
    u->flags |= URL_FLAG_STREAMED;

    return u;
}
