/**
    Copyright (C) 2001  Michael Ahlberg, Måns Rullgård

    Permission is hereby granted, free of charge, to any person
    obtaining a copy of this software and associated documentation
    files (the "Software"), to deal in the Software without
    restriction, including without limitation the rights to use, copy,
    modify, merge, publish, distribute, sublicense, and/or sell copies
    of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be
    included in all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
    EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
    NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
    HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
    WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.
**/

#include <stdlib.h>
#include <stdio.h>
#include <readline/readline.h>
#include <readline/history.h>
#include "commands.h"
#include <tclist.h>
#include <string.h>
#include <stdlib.h>
#include "shell_tc2.h"

int prompt;
char *prompt_str;
tclist_t *sh_prompt;

extern int sh_reg_prompt(char *p)
{
    tclist_unshift(sh_prompt, prompt_str);
    prompt_str = strdup(p);
    return 0;
}

extern int sh_unreg_prompt(void)
{
    char *p;
    p = tclist_shift(sh_prompt);
    if(p==NULL) {
	return -1;
    }
    free(prompt_str);
    prompt_str=p;
    return 0;
}

/* Read a string, and return a pointer to it.  Returns NULL on EOF. */
char *rl_gets(char **line_read)
{
    /* If the buffer has already been allocated, return the memory
       to the free pool. */
    if (*line_read)
    {
	free (*line_read);
	*line_read = (char *)NULL;
    }
    
    /* Get a line from the user. */
    prompt=1;
    *line_read = readline (prompt_str);
    prompt=0;

    /* If the line has any text in it, save it on the history. */
    if (*line_read && **line_read)
	add_history (*line_read);
    
    return (*line_read);
}

char *command_generator (const char *text, int state)
{
    static int len;
    char *name;
    static tclist_item_t *current;
    static command *cmd;

    if(state == 0){
	current=NULL;
	cmd = tclist_next(cmd_list, &current);
	len = strlen (text);
    }

    while(cmd!=NULL){
	name = cmd->name;
	cmd = tclist_next(cmd_list, &current);

	if (strncmp (name, text, len) == 0){
	    /* tclist_unlock(current); */
	    return (strdup(name));
	}
    }

    /* If no names matched, then return NULL. */
    return ((char *)NULL);
}

char **shell_completion (char *text, int start, int end)
{
    char **matches;
    matches = (char **)NULL;
    if (start == 0)
	matches = rl_completion_matches (text, command_generator);
    return (matches);
}

int init_rl(void)
{
    sh_prompt=tclist_new(TC_LOCK_SLOPPY);
    read_history(tcsh_hist_file);
    rl_attempted_completion_function = (CPPFunction *)shell_completion;
    return 0;
}

int uninit_rl(void)
{
    /* FIXME: remove old prompts */
    write_history(tcsh_hist_file);
    if(prompt){
	system("stty sane");
	printf("\n");
    }
    return 0;
}
