/**
    Copyright (C) 2002  Michael Ahlberg, Måns Rullgård

    Permission is hereby granted, free of charge, to any person
    obtaining a copy of this software and associated documentation
    files (the "Software"), to deal in the Software without
    restriction, including without limitation the rights to use, copy,
    modify, merge, publish, distribute, sublicense, and/or sell copies
    of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be
    included in all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
    EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
    NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
    HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
    WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.
**/

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <alloca.h>
#include <tctypes.h>
#include <tcstring.h>
#include <url_tc2.h>

#define DEFAULT_HANDLER "file"


extern url_t *
url_open(char *url, char *mode)
{
    char *type;
    char *p;
    char *class;
    int tp_len;
    url_open_t open_fn;

    if((p = strchr(url, ':'))){
	tp_len = p - url;
	type = alloca(tp_len + 1);
	memcpy(type, url, tp_len);
	type[tp_len] = 0;
    } else {
	tp_len = strlen(DEFAULT_HANDLER);
	type = DEFAULT_HANDLER;
    }

    class = alloca(4 + tp_len + 1); /* strlen("URL/") = 4*/
    strcpy(class, "URL/");
    strcpy(class+4, type);
    open_fn = tc2_get_symbol(class, "open");

    return open_fn? open_fn(url, mode): NULL;
}


extern int
url_read(void *buf, size_t size, size_t count, url_t *stream)
{
    return stream->read(buf, size, count, stream);
}


extern int
url_write(void *buf, size_t size, size_t count, url_t *stream)
{
    return stream->write(buf, size, count, stream);
}


extern int
url_close(url_t *stream)
{
    return stream->close(stream);
}


extern int
url_seek(url_t *stream, int64_t offset, int how)
{
    return stream->seek(stream, offset, how);
}


extern uint64_t
url_tell(url_t *stream)
{
    return stream->tell(stream);
}


extern int
url_getc(url_t *u)
{
    u_char c;
    int r;

    r = u->read(&c, 1, 1, u);
    if(r != 1)
	return -1;

    return c;
}


extern char *
url_gets(char *s, int c, url_t *u)
{
    int i, r;

    for(i = 0; i < c - 1;){
	r = u->read(s + i, 1, 1, u);
	if(r <= 0)
	    break;
	if(s[i++] == '\n')
	    break;
    }

    if(i)
	s[i] = 0;

    return i? s: NULL;
}
